// graphics primitives

SDL_Surface* loadImage(char* filename);

void applySurface(int x, int y, SDL_Surface* source, SDL_Surface* destination, SDL_Rect* clip);

// SDL sound


#ifdef SOUND_ON
Mix_Music* loadBGM(char* filename);

Mix_Chunk* loadSFX(char* filename);
#endif
// world

void renderBackground();

void renderForeground();

void interact();

void kListen(int* whichKaos);

// system

int init(int argc, char* args[]);

void writeSpellBook();

void toggleFullscreen();

void timeDilation();

void cleanup();

void intro();

void mainmenu();

void pausemenu();
